@@ -56,8 +56,10 @@ multiple : {{"layer", "stack", "fill"}}
     """,
     log_scale="""
 log_scale : bool or number, or pair of bools or numbers
-    Set a log scale on the data axis (or axes, with bivariate data) with the
-    given base (default 10), and evaluate the KDE in log space.
+    Set axis scale(s) to log. A single value sets the data axis for univariate
+    distributions and both axes for bivariate distributions. A pair of values
+    sets each axis independently. Numeric values are interpreted as the desired
+    base (default 10). If `False`, defer to the existing Axes scale.
     """,
     legend="""
 legend : bool
@@ -170,9 +172,9 @@ class _DistributionPlotter(VectorPlotter):
                 kws.setdefault("edgecolor", to_rgba(color, 1))
         elif element == "bars":
             kws["facecolor"] = "none"
-            kws["edgecolor"] = to_rgba(color, 1)
+            kws["edgecolor"] = to_rgba(color, alpha)
         else:
-            kws["color"] = color
+            kws["color"] = to_rgba(color, alpha)
         return kws
 
     def _quantile_to_level(self, data, quantile):
@@ -210,8 +212,16 @@ class _DistributionPlotter(VectorPlotter):
         return discrete
 
     def _resolve_multiple(self, curves, multiple):
+        """Modify the density data structure to handle multiple densities."""
+
+        # Default baselines have all densities starting at 0
+        baselines = {k: np.zeros_like(v) for k, v in curves.items()}
+
+        # TODO we should have some central clearinghouse for checking if any
+        # "grouping" (terminnology?) semantics have been assigned
+        if "hue" not in self.variables:
+            return curves, baselines
 
-        # Modify the density data structure to handle multiple densities
         if multiple in ("stack", "fill"):
 
             # Setting stack or fill means that the curves share a
@@ -246,11 +256,6 @@ class _DistributionPlotter(VectorPlotter):
                                            .shift(1, axis=1)
                                            .fillna(0))
 
-        else:
-
-            # All densities will start at 0
-            baselines = {k: np.zeros_like(v) for k, v in curves.items()}
-
         if multiple == "dodge":
 
             # Account for the unique semantic (non-faceting) levels
@@ -278,6 +283,7 @@ class _DistributionPlotter(VectorPlotter):
         common_grid,
         estimate_kws,
         log_scale,
+        warn_singular=True,
     ):
 
         # Initialize the estimator object
@@ -302,8 +308,12 @@ class _DistributionPlotter(VectorPlotter):
 
             observation_variance = observations.var()
             if math.isclose(observation_variance, 0) or np.isnan(observation_variance):
-                msg = "Dataset has 0 variance; skipping density estimate."
-                warnings.warn(msg, UserWarning)
+                msg = (
+                    "Dataset has 0 variance; skipping density estimate. "
+                    "Pass `warn_singular=False` to disable this warning."
+                )
+                if warn_singular:
+                    warnings.warn(msg, UserWarning)
                 continue
 
             # Extract the weights for this subset of observations
@@ -391,7 +401,7 @@ class _DistributionPlotter(VectorPlotter):
 
             if common_bins:
                 all_observations = all_data[self.data_variable]
-                estimator.define_bin_edges(
+                estimator.define_bin_params(
                     all_observations,
                     weights=all_data.get("weights", None),
                 )
@@ -399,13 +409,6 @@ class _DistributionPlotter(VectorPlotter):
         else:
             common_norm = False
 
-        # Turn multiple off if no hue or if hue exists but is redundent with faceting
-        facet_vars = [self.variables.get(var, None) for var in ["row", "col"]]
-        if "hue" not in self.variables:
-            multiple = None
-        elif self.variables["hue"] in facet_vars:
-            multiple = None
-
         # Estimate the smoothed kernel densities, for use later
         if kde:
             # TODO alternatively, clip at min/max bins?
@@ -418,6 +421,7 @@ class _DistributionPlotter(VectorPlotter):
                 common_bins,
                 kde_kws,
                 log_scale,
+                warn_singular=False,
             )
 
         # First pass through the data to compute the histograms
@@ -450,9 +454,12 @@ class _DistributionPlotter(VectorPlotter):
                 edges = np.power(10, edges)
 
             # Pack the histogram data and metadata together
+            orig_widths = np.diff(edges)
+            widths = shrink * orig_widths
+            edges = edges[:-1] + (1 - shrink) / 2 * orig_widths
             index = pd.MultiIndex.from_arrays([
-                pd.Index(edges[:-1], name="edges"),
-                pd.Index(np.diff(edges) * shrink, name="widths"),
+                pd.Index(edges, name="edges"),
+                pd.Index(widths, name="widths"),
             ])
             hist = pd.Series(heights, index=index, name="heights")
 
@@ -517,12 +524,16 @@ class _DistributionPlotter(VectorPlotter):
                     scout.remove()
 
         # Default alpha should depend on other parameters
-        if multiple == "layer":
-            default_alpha = .5 if element == "bars" else .25
-        elif kde:
-            default_alpha = .5
+        if fill:
+            # Note: will need to account for other grouping semantics if added
+            if "hue" in self.variables and multiple == "layer":
+                default_alpha = .5 if element == "bars" else .25
+            elif kde:
+                default_alpha = .5
+            else:
+                default_alpha = .75
         else:
-            default_alpha = .75
+            default_alpha = 1
         alpha = plot_kws.pop("alpha", default_alpha)  # TODO make parameter?
 
         hist_artists = []
@@ -551,9 +562,8 @@ class _DistributionPlotter(VectorPlotter):
                 # Use matplotlib bar plotting
 
                 plot_func = ax.bar if self.data_variable == "x" else ax.barh
-                move = .5 * (1 - shrink)
                 artists = plot_func(
-                    hist["edges"] + move,
+                    hist["edges"],
                     hist["heights"] - bottom,
                     hist["widths"],
                     bottom,
@@ -644,6 +654,18 @@ class _DistributionPlotter(VectorPlotter):
 
             # Now we handle linewidth, which depends on the scaling of the plot
 
+            # We will base everything on the minimum bin width
+            hist_metadata = pd.concat([
+                # Use .items for generality over dict or df
+                h.index.to_frame() for _, h in histograms.items()
+            ]).reset_index(drop=True)
+            thin_bar_idx = hist_metadata["widths"].idxmin()
+            binwidth = hist_metadata.loc[thin_bar_idx, "widths"]
+            left_edge = hist_metadata.loc[thin_bar_idx, "edges"]
+
+            # Set initial value
+            default_linewidth = math.inf
+
             # Loop through subsets based only on facet variables
             for sub_vars, _ in self.iter_data():
 
@@ -653,16 +675,10 @@ class _DistributionPlotter(VectorPlotter):
                 # Innocuous in other cases?
                 ax.autoscale_view()
 
-                # We will base everything on the minimum bin width
-                hist_metadata = [h.index.to_frame() for _, h in histograms.items()]
-                binwidth = min([
-                    h["widths"].min() for h in hist_metadata
-                ])
-
-                # Convert binwidtj from data coordinates to pixels
-                pts_x, pts_y = 72 / ax.figure.dpi * (
-                    ax.transData.transform([binwidth, binwidth])
-                    - ax.transData.transform([0, 0])
+                # Convert binwidth from data coordinates to pixels
+                pts_x, pts_y = 72 / ax.figure.dpi * abs(
+                    ax.transData.transform([left_edge + binwidth] * 2)
+                    - ax.transData.transform([left_edge] * 2)
                 )
                 if self.data_variable == "x":
                     binwidth_points = pts_x
@@ -671,24 +687,24 @@ class _DistributionPlotter(VectorPlotter):
 
                 # The relative size of the lines depends on the appearance
                 # This is a provisional value and may need more tweaking
-                default_linewidth = .1 * binwidth_points
+                default_linewidth = min(.1 * binwidth_points, default_linewidth)
 
-                # Set the attributes
-                for bar in hist_artists:
+            # Set the attributes
+            for bar in hist_artists:
 
-                    # Don't let the lines get too thick
-                    max_linewidth = bar.get_linewidth()
-                    if not fill:
-                        max_linewidth *= 1.5
+                # Don't let the lines get too thick
+                max_linewidth = bar.get_linewidth()
+                if not fill:
+                    max_linewidth *= 1.5
 
-                    linewidth = min(default_linewidth, max_linewidth)
+                linewidth = min(default_linewidth, max_linewidth)
 
-                    # If not filling, don't let lines dissapear
-                    if not fill:
-                        min_linewidth = .5
-                        linewidth = max(linewidth, min_linewidth)
+                # If not filling, don't let lines dissapear
+                if not fill:
+                    min_linewidth = .5
+                    linewidth = max(linewidth, min_linewidth)
 
-                    bar.set_linewidth(linewidth)
+                bar.set_linewidth(linewidth)
 
         # --- Finalize the plot ----
 
@@ -734,7 +750,7 @@ class _DistributionPlotter(VectorPlotter):
         if set(self.variables) - {"x", "y"}:
             all_data = self.comp_data.dropna()
             if common_bins:
-                estimator.define_bin_edges(
+                estimator.define_bin_params(
                     all_data["x"],
                     all_data["y"],
                     all_data.get("weights", None),
@@ -752,7 +768,7 @@ class _DistributionPlotter(VectorPlotter):
             )
             full_heights.append(sub_heights)
 
-        common_color_norm = "hue" not in self.variables or common_norm
+        common_color_norm = not set(self.variables) - {"x", "y"} or common_norm
 
         if pthresh is not None and common_color_norm:
             thresh = self._quantile_to_level(full_heights, pthresh)
@@ -762,7 +778,7 @@ class _DistributionPlotter(VectorPlotter):
             if pmax is not None:
                 vmax = self._quantile_to_level(full_heights, pmax)
             else:
-                vmax = plot_kws.pop("vmax", np.max(full_heights))
+                vmax = plot_kws.pop("vmax", max(map(np.max, full_heights)))
         else:
             vmax = None
 
@@ -879,6 +895,7 @@ class _DistributionPlotter(VectorPlotter):
         multiple,
         common_norm,
         common_grid,
+        warn_singular,
         fill,
         legend,
         estimate_kws,
@@ -914,9 +931,10 @@ class _DistributionPlotter(VectorPlotter):
             common_grid,
             estimate_kws,
             log_scale,
+            warn_singular,
         )
 
-        # Note: raises when no hue and multiple != layer. A problem?
+        # Adjust densities based on the `multiple` rule
         densities, baselines = self._resolve_multiple(densities, multiple)
 
         # Control the interaction with autoscaling by defining sticky_edges
@@ -956,8 +974,13 @@ class _DistributionPlotter(VectorPlotter):
                     scout.remove()
 
         plot_kws.pop("color", None)
-
-        default_alpha = .25 if multiple == "layer" else .75
+        if fill:
+            if multiple == "layer":
+                default_alpha = .25
+            else:
+                default_alpha = .75
+        else:
+            default_alpha = 1
         alpha = plot_kws.pop("alpha", default_alpha)  # TODO make parameter?
 
         # Now iterate through the subsets and draw the densities
@@ -1041,6 +1064,7 @@ class _DistributionPlotter(VectorPlotter):
         color,
         legend,
         cbar,
+        warn_singular,
         cbar_ax,
         cbar_kws,
         estimate_kws,
@@ -1074,8 +1098,12 @@ class _DistributionPlotter(VectorPlotter):
             # Check that KDE will not error out
             variance = observations[["x", "y"]].var()
             if any(math.isclose(x, 0) for x in variance) or variance.isna().any():
-                msg = "Dataset has 0 variance; skipping density estimate."
-                warnings.warn(msg, UserWarning)
+                msg = (
+                    "Dataset has 0 variance; skipping density estimate. "
+                    "Pass `warn_singular=False` to disable this warning."
+                )
+                if warn_singular:
+                    warnings.warn(msg, UserWarning)
                 continue
 
             # Estimate the density of observations at this level
@@ -1243,6 +1271,12 @@ class _DistributionPlotter(VectorPlotter):
             if "hue" in self.variables:
                 artist_kws["color"] = self._hue_map(sub_vars["hue"])
 
+            # Return the data variable to the linear domain
+            # This needs an automatic solution; see GH2409
+            if self._log_scaled(self.data_variable):
+                vals = np.power(10, vals)
+                vals[0] = -np.inf
+
             # Work out the orientation of the plot
             if self.data_variable == "x":
                 plot_args = vals, stat
@@ -1291,7 +1325,7 @@ class _DistributionPlotter(VectorPlotter):
             kws["color"] = kws.pop("color", scout.get_color())
             scout.remove()
 
-        for sub_vars, sub_data, in self.iter_data():
+        for sub_vars, sub_data, in self.iter_data(from_comp_data=True):
 
             ax = self._get_axes(sub_vars)
 
@@ -1328,6 +1362,11 @@ class _DistributionPlotter(VectorPlotter):
         vector = sub_data[var]
         n = len(vector)
 
+        # Return data to linear domain
+        # This needs an automatic solution; see GH2409
+        if self._log_scaled(var):
+            vector = np.power(10, vector)
+
         # We'll always add a single collection with varying colors
         if "hue" in self.variables:
             colors = self._hue_map(sub_data["hue"])
@@ -1611,6 +1650,9 @@ def kdeplot(
     # Renamed params
     data=None, data2=None,
 
+    # New in v0.12
+    warn_singular=True,
+
     **kwargs,
 ):
 
@@ -1731,6 +1773,7 @@ def kdeplot(
             common_grid=common_grid,
             fill=fill,
             legend=legend,
+            warn_singular=warn_singular,
             estimate_kws=estimate_kws,
             **plot_kws,
         )
@@ -1744,6 +1787,7 @@ def kdeplot(
             thresh=thresh,
             legend=legend,
             color=color,
+            warn_singular=warn_singular,
             cbar=cbar,
             cbar_ax=cbar_ax,
             cbar_kws=cbar_kws,
@@ -1839,6 +1883,9 @@ fill : bool or None
     If True, fill in the area under univariate density curves or between
     bivariate contours. If None, the default depends on ``multiple``.
 {params.core.data}
+warn_singular : bool
+    If True, issue a warning when trying to estimate the density of data
+    with zero variance.
 kwargs
     Other keyword arguments are passed to one of the following matplotlib
     functions:
@@ -2170,8 +2217,8 @@ def displot(
             p.variables[var] = f"_{var}_"
 
     # Adapt the plot_data dataframe for use with FacetGrid
-    data = p.plot_data.rename(columns=p.variables)
-    data = data.loc[:, ~data.columns.duplicated()]
+    grid_data = p.plot_data.rename(columns=p.variables)
+    grid_data = grid_data.loc[:, ~grid_data.columns.duplicated()]
 
     col_name = p.variables.get("col", None)
     row_name = p.variables.get("row", None)
@@ -2180,7 +2227,7 @@ def displot(
         facet_kws = {}
 
     g = FacetGrid(
-        data=data, row=row_name, col=col_name,
+        data=grid_data, row=row_name, col=col_name,
         col_wrap=col_wrap, row_order=row_order,
         col_order=col_order, height=height,
         aspect=aspect,
@@ -2299,6 +2346,21 @@ def displot(
     g.set_titles()
     g.tight_layout()
 
+    if data is not None and (x is not None or y is not None):
+        if not isinstance(data, pd.DataFrame):
+            data = pd.DataFrame(data)
+        g.data = pd.merge(
+            data,
+            g.data[g.data.columns.difference(data.columns)],
+            left_index=True,
+            right_index=True,
+        )
+    else:
+        wide_cols = {
+            k: f"_{k}_" if v is None else v for k, v in p.variables.items()
+        }
+        g.data = p.plot_data.rename(columns=wide_cols)
+
     return g
 
 
